#include <TxtGlue.h>
#include "progress.h"

struct Progress {
	FormPtr prevForm;
};



#define PBAR_LEFT		2
#define PBAR_TOP		24
#define PBAR_WIDTH		122
#define PBAR_HEIGHT		8




static void progressPrvPbarDraw(UInt32 num, UInt32 of)
{
	RectangleType r, e;
	
	r.topLeft.x = PBAR_LEFT;
	r.topLeft.y = PBAR_TOP;
	r.extent.x = PBAR_WIDTH;
	r.extent.y = PBAR_HEIGHT;
	
	WinDrawRectangleFrame(rectangleFrame, &r);
	RctInsetRectangle(&r, 1);
	WinEraseRectangleFrame(rectangleFrame, &r);
	RctCopyRectangle(&r, &e);
	r.extent.x = (unsigned long long)num * r.extent.x / of;
	e.extent.x -= r.extent.x;
	e.topLeft.x += r.extent.x;
	WinDrawRectangle(&r, 0);
	WinEraseRectangle(&e, 0);
}

//show a string at coordinates. if it does not fit, ellipsize it with 1/3 shown of start, 2/3 of end
static void progressPrvShowString(const char *str, Int16 x, Int16 y, UInt16 width)
{
	UInt16 len = StrLen(str), fullWidth = FntCharsWidth(str, len), spacerWidth = FntCharWidth(chrEllipsis);
	char ellipsis = TxtGlueGetHorizEllipsisChar();
	
	if (fullWidth <= width)
		WinDrawChars(str, len, x, y);
	else if (width > spacerWidth) {		//we know it will not fit
		
		UInt16 spaceAvail = width - spacerWidth, spaceBefore = spaceAvail / 3, spaceAfter = spaceAvail - spaceBefore, ofst = 0, chrSz, chrWidth;
		UInt32 charStart = 0, firstPrintedCharSecondPart;
		WChar chr;
		
		//print first part
		while (1) {
			
			chrSz = TxtGetNextChar(str, charStart, &chr);
			chrWidth = TxtCharWidth(chr);
			if (spaceBefore - ofst < chrWidth)
				break;
			WinDrawChars(str + charStart, chrSz, x + ofst, y);
			charStart += chrSz;
			ofst += chrWidth;
		}
		
		//print the ellipsis
		WinDrawChars(&ellipsis, 1, x + ofst, y);
		ofst += spacerWidth;
		
		//recalc how much space we DO have, reuse vars
		spaceAfter = width - ofst;
		x += ofst;
		ofst = 0;
		
		//sort out how much of the last part we can print..backwards
		chr = TxtCharBounds(str, len - 1, &charStart, &firstPrintedCharSecondPart);
		chrSz = firstPrintedCharSecondPart - charStart;
		while(1) {
			chrWidth = TxtCharWidth(chr);
			
			if (spaceAfter - ofst < chrWidth)
				break;
			
			ofst += chrWidth;
			firstPrintedCharSecondPart -= chrSz;
			
			chrSz = TxtGetPreviousChar(str, firstPrintedCharSecondPart, &chr);
		}
		
		//print it
		WinDrawChars(str + firstPrintedCharSecondPart, len - firstPrintedCharSecondPart, x, y);
	}
}

struct Progress* progressStart(const char *name)
{
	struct Progress *p = MemPtrNew(sizeof(struct Progress));
	FormPtr fp;
	
	p->prevForm = FrmGetActiveForm();
	fp = FrmInitForm(1004);
	
	FrmSetActiveForm(fp);
	FrmDrawForm(fp);
	
	progressPrvShowString(name, PBAR_LEFT, 0, PBAR_WIDTH);
	
	return p;
}

void progressUpdate(struct Progress *p, UInt32 done, UInt32 of)
{
	UInt16 width, len;
	char x[22];
	
	if (done > of)
		done = of;
	
	progressPrvPbarDraw(done, of);
	StrCopy(x, "   ");
	StrIToA(x + 3, ((unsigned long long)done * 100 + of / 2) / of);
	StrCat(x, "%   ");
	
	len = StrLen(x);
	width = FntCharsWidth(x, len);
	WinDrawChars(x, len, PBAR_LEFT + (PBAR_WIDTH - width) / 2, 12);
}

void progressFinish(struct Progress *p)
{
	FormPtr fp = FrmGetActiveForm();
	
	FrmEraseForm(fp);
	FrmSetActiveForm(p->prevForm);
	FrmDeleteForm(fp);
	MemPtrFree(p);
}

