#include <PalmOS.h>



struct PrefMapping {
	UInt8 offset		: 6;
	UInt8 sizeLg2M1		: 2;	//0 = no such pref
};

static const struct PrefMapping mPrefs[] = {
	[prefVersion] = {0, 2},
	[prefCountry] = {2, 1},
	[prefDateFormat] = {3, 1},
	[prefLongDateFormat] = {4, 1},
	[prefWeekStartDay] = {5, 1},
	[prefTimeFormat] = {6, 1},
	[prefNumberFormat] = {7, 1},
	[prefAutoOffDuration] = {8, 1},
	[prefSysSoundLevelV20] = {9, 1},
	[prefAlarmSoundLevelV20] = {10, 1},
	[prefHidePrivateRecordsV33] = {11, 1},
	[prefDeviceLocked] = {12, 1},
	[prefSysBatteryKind] = {16, 0},
};


static UInt32 pPrefGetPreference(SystemPreferencesChoice choice)
{
	SystemPreferencesTypeV10 prefs;
	struct PrefMapping m;
	UInt8 ofst, sz;
	
	if (choice > sizeof(mPrefs) / sizeof(*mPrefs))
		return 0;
	m = mPrefs[choice];
	
	ofst = m.offset;
	sz = m.sizeLg2M1;
	
	PrefGetPreferences((SystemPreferencesPtr)&prefs);
	
	switch (sz) {
		case 0:
		default:
			return 0;
		
		case 1:
			return ((UInt8*)&prefs)[ofst];
		
		case 2:
			return *(UInt16*)(((UInt8*)&prefs) + ofst);
	}
}

static void pPrefSetPreference(SystemPreferencesChoice choice, UInt32 value)
{
	SystemPreferencesTypeV10 prefs;
	struct PrefMapping m;
	UInt8 ofst, sz;
	
	if (choice > sizeof(mPrefs) / sizeof(*mPrefs))
		return;
	m = mPrefs[choice];
	
	ofst = m.offset;
	sz = m.sizeLg2M1;
	
	PrefGetPreferences((SystemPreferencesPtr)&prefs);
	
	switch (sz) {
		case 1:
			((UInt8*)&prefs)[ofst] = value;
			break;
		
		case 2:
			*(UInt16*)(((UInt8*)&prefs) + ofst) = value;
			break;
	}
	
	PrefSetPreferences((SystemPreferencesPtr)&prefs);
}

void __Startup__(void)
{
	UInt32 ver;
		
	if (errNone == FtrGet(sysFtrCreator, sysFtrNumROMVersion, &ver) && ver < sysMakeROMVersion(2, 0, 0, 0, 0)) {
		
		SysSetTrapAddress(sysTrapPrefGetPreference, pPrefGetPreference);
		SysSetTrapAddress(sysTrapPrefSetPreference, pPrefSetPreference);
		
		MemHandleLock(DmGet1Resource('extn', 4));
	}
}
